@@ -25,11 +25,6 @@
 #include <sys/stat.h>
 #include <sys/fcntl.h>
 
-#include <camel/camel-mime-message.h>
-#include <camel/camel-multipart.h>
-#include <camel/camel-stream-mem.h>
-#include <camel/camel-file-utils.h>
-
 #include "e-cal-backend-exchange-calendar.h"
 
 #include "e2k-cal-utils.h"
@@ -46,14 +41,21 @@ struct ECalBackendExchangeCalendarPrivate {
 	int dummy;
 };
 
+enum {
+	EX_NO_RECEIPTS = 0,
+	EX_DELIVERED_RECEIPTS,
+	EX_READ_AND_DELIVERED,
+	EX_ALL
+};
+
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_EXCHANGE
 static ECalBackendExchange *parent_class = NULL;
 
-#define d(x) (x)
+#define d(x)
 
 static ECalBackendSyncStatus modify_object_with_href (ECalBackendSync *backend, EDataCal *cal, const char *calobj, CalObjModType mod, char **old_object, const char *href);
 
-static GSList * get_attachment (ECalBackendExchange *cbex, const char *uid, const char *body, int len);
+gboolean check_for_send_options (icalcomponent *icalcomp, E2kProperties *props);
 
 static void
 add_timezones_from_comp (ECalBackendExchange *cbex, icalcomponent *icalcomp)
@@ -118,11 +120,14 @@ add_vevent (ECalBackendExchange *cbex,
 			icalcomponent_set_dtend (icalcomp, itt);
 		}
 
-		if (!strcmp (x_name, "X-MICROSOFT-CDO-BUSYSTATUS") && !transp) {
+		if (!strcmp (x_name, "X-MICROSOFT-CDO-BUSYSTATUS")) {
 			/* It seems OWA sometimes doesn't set the
 			 * TRANSP property, so set it from the busy
 			 * status.
 			 */
+			if (transp)
+				icalcomponent_remove_property (icalcomp,transp);
+
 			if (!strcmp (x_val, "BUSY"))
 				transp = icalproperty_new_transp (ICAL_TRANSP_OPAQUE);
 			else if (!strcmp (x_val, "FREE"))
@@ -164,12 +169,13 @@ add_vevent (ECalBackendExchange *cbex,
 
 static gboolean
 add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod, 
-	  const char *uid, const char *body, int len)
+	  const char *uid, const char *body, int len, int receipts)
 {
 	const char *start, *end;
 	char *ical_body;
 	icalcomponent *icalcomp, *subcomp, *new_comp;
 	icalcomponent_kind kind;
+	icalproperty *icalprop;
 	ECalComponent *ecomp;
 	GSList *attachment_list = NULL;
 	gboolean status;
@@ -178,7 +184,6 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 	if (uid)
 		attachment_list = get_attachment (cbex, uid, body, len);
 
-
 	start = g_strstr_len (body, len, "\nBEGIN:VCALENDAR");
 	if (!start)
 		return FALSE;
@@ -196,11 +201,15 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 
 	kind = icalcomponent_isa (icalcomp);
 	if (kind == ICAL_VEVENT_COMPONENT) {
+		if (receipts) {
+			icalprop = icalproperty_new_x (g_strdup (GINT_TO_POINTER (receipts)));
+			icalproperty_set_x_name (icalprop, "X-EVOLUTION-OPTIONS-TRACKINFO");
+			icalcomponent_add_property (icalcomp, icalprop);
+		}
 		if (attachment_list) {
 			ecomp = e_cal_component_new ();
 			e_cal_component_set_icalcomponent (ecomp, icalcomponent_new_clone (icalcomp));
 			e_cal_component_set_attachment_list (ecomp, attachment_list);
-			icalcomponent_free (icalcomp);
 			icalcomp = icalcomponent_new_clone (e_cal_component_get_icalcomponent (ecomp));
 			g_object_unref (ecomp);
 		}
@@ -217,14 +226,17 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 	subcomp = icalcomponent_get_first_component (
 		icalcomp, ICAL_VEVENT_COMPONENT);
 	while (subcomp) {
-		new_comp = icalcomponent_new_clone (subcomp);
+		if (uid && !strcmp (uid, icalcomponent_get_uid (subcomp)) && attachment_list) {
+			ecomp = e_cal_component_new ();
+			e_cal_component_set_icalcomponent (ecomp, icalcomponent_new_clone (subcomp));
+			e_cal_component_set_attachment_list (ecomp, attachment_list);
+			new_comp = icalcomponent_new_clone (e_cal_component_get_icalcomponent (ecomp));
+			g_object_unref (ecomp);
+		} else {
+			new_comp = icalcomponent_new_clone (subcomp);
+		}
+
 		if (new_comp) {
-			/*
-			if (attach_len) {
-				iattach = icalattach_new_from_data (attach_data, g_free, NULL);
-				attachment_list = g_slist_append (attachment_list, iattach);
-				e_cal_component_set_attachment_list (new_comp, attachment_list);
-			} */
 			add_vevent (cbex, href, lastmod, new_comp);
 			icalcomponent_free (new_comp);
 		}
@@ -236,87 +248,19 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 	return TRUE;
 }
 
-static GSList * 
-get_attachment (ECalBackendExchange *cbex, const char *uid, 
-			const char *body, int len)
-{
-	CamelStream *stream;
-	CamelMimeMessage *msg;
-	CamelDataWrapper *content;
-	CamelMultipart *multipart;
-	CamelMimePart *part;
-	const char *filename = NULL;
-	char *attach_filename, *attach_file, *attach_file_url;
-	int fd;
-	int i;
-	GSList *list = NULL;
-	unsigned char *attach_data;
-
-	stream = camel_stream_mem_new_with_buffer (body, len);
-	msg = camel_mime_message_new ();
-	camel_data_wrapper_construct_from_stream (CAMEL_DATA_WRAPPER (msg), stream);
-	camel_object_unref (stream);
-
-	content = camel_medium_get_content_object (CAMEL_MEDIUM (msg));
-	if (CAMEL_IS_MULTIPART (content)) {
-		multipart = (CamelMultipart *)content;
-		content = NULL;
-	
-		for (i = 0; i < (int)camel_multipart_get_number (multipart); i++) {
-			part = camel_multipart_get_part (multipart, i);
-			filename = camel_mime_part_get_filename (part);
-			if (filename) {
-				content = camel_medium_get_content_object (CAMEL_MEDIUM (part));
-				break;
-			}
-		}
-	}
-
-	if (content) {
-		CamelStreamMem *stream_mem;
-		
-		stream = camel_stream_mem_new ();
-		stream_mem = (CamelStreamMem *)stream;
-	
-		camel_data_wrapper_decode_to_stream (content, stream);
-		attach_data = g_memdup (stream_mem->buffer->data, stream_mem->buffer->len);
-
-		// Attach
-		attach_filename = g_strdup_printf ("%s-%s", uid, filename);
-		attach_file = e_folder_exchange_get_storage_file (cbex->folder, attach_filename);
-		g_free (attach_filename);
-
-		fd = open (attach_file, O_RDWR|O_CREAT|O_TRUNC, 0600);
-		if (fd < 0) {
-			d(printf ("could not open file for creating attachment file locally\n"));
-		} else {
-			if (camel_write (fd, attach_data, stream_mem->buffer->len) < 0)
-				d(printf ("camel write to attach file failed\n"));
-		}
-		attach_file_url = g_strdup_printf ("file://%s", attach_file);
-		list = g_slist_append (list, g_strdup (attach_file_url));
-
-		close (fd);
-		g_free (attach_file_url);
-		g_free (attach_file);
-		
-		camel_object_unref (stream);
-	}
-	
-	camel_object_unref (msg);
-	return list;
-}
-
 static const char *event_properties[] = {
 	E2K_PR_CALENDAR_UID,
 	E2K_PR_DAV_LAST_MODIFIED,
 	E2K_PR_HTTPMAIL_HAS_ATTACHMENT,
+	PR_READ_RECEIPT_REQUESTED,
+	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED	
 };
 static const int n_event_properties = G_N_ELEMENTS (event_properties);
 
 static const char *new_event_properties[] = {
 	PR_INTERNET_CONTENT,
-	E2K_PR_HTTPMAIL_HAS_ATTACHMENT,
+	PR_READ_RECEIPT_REQUESTED,
+	PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED	
 };
 static const int n_new_event_properties = G_N_ELEMENTS (new_event_properties);
 
@@ -329,11 +273,11 @@ get_changed_events (ECalBackendExchange *cbex, const char *since)
 	E2kRestriction *rn;
 	E2kResultIter *iter;
 	E2kResult *result;
-	const char *prop, *uid, *modtime, *attach_prop;
+	const char *prop, *uid, *modtime, *attach_prop, *receipts;
 	char *body;
 	guint status;
 	E2kContext *ctx;
-	int i;
+	int i, status_tracking = EX_NO_RECEIPTS;
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex), SOUP_STATUS_CANCELLED);
 
@@ -385,6 +329,21 @@ get_changed_events (ECalBackendExchange *cbex, const char *since)
 		attach_prop = e2k_properties_get_prop (result->props, 
 						E2K_PR_HTTPMAIL_HAS_ATTACHMENT);
 
+		receipts = e2k_properties_get_prop (result->props,
+						PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED);
+		if (receipts && atoi (receipts))
+			status_tracking = EX_DELIVERED_RECEIPTS;
+	
+		receipts = NULL;
+		receipts = e2k_properties_get_prop (result->props,
+						PR_READ_RECEIPT_REQUESTED);
+		if (receipts && atoi (receipts)) {
+			if (status_tracking == EX_DELIVERED_RECEIPTS)
+				status_tracking = EX_ALL;
+			else
+				status_tracking = EX_READ_AND_DELIVERED;
+		}
+
 		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href)) {
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			g_hash_table_insert (modtimes, g_strdup (result->href),
@@ -427,6 +386,7 @@ get_changed_events (ECalBackendExchange *cbex, const char *since)
 
 	while ((result = e2k_result_iter_next (iter))) {
 		GByteArray *ical_data;
+		status_tracking = EX_NO_RECEIPTS;
 
 		ical_data = e2k_properties_get_prop (result->props, PR_INTERNET_CONTENT);
 		if (!ical_data) {
@@ -434,14 +394,28 @@ get_changed_events (ECalBackendExchange *cbex, const char *since)
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			continue;
 		}
+		receipts = e2k_properties_get_prop (result->props,
+					PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED);
+		if (receipts && atoi (receipts))
+			status_tracking = EX_DELIVERED_RECEIPTS;
+	
+		receipts = NULL;
+		receipts = e2k_properties_get_prop (result->props,
+					PR_READ_RECEIPT_REQUESTED);
+		if (receipts && atoi (receipts)) {
+			if (status_tracking == EX_DELIVERED_RECEIPTS)
+				status_tracking = EX_ALL;
+			else
+				status_tracking = EX_READ_AND_DELIVERED;
+		}
+
 		modtime = g_hash_table_lookup (modtimes, result->href);
 		uid = g_hash_table_lookup (attachments, result->href);
 		/* The icaldata already has the attachment. So no need to
 			re-fetch it from the server. */
 		add_ical (cbex, result->href, modtime, uid,
-			  ical_data->data, ical_data->len);
+			  ical_data->data, ical_data->len, status_tracking);
 	}
-	// g_byte_array_free (ical_data);
 	status = e2k_result_iter_free (iter);
 
 	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
@@ -472,7 +446,7 @@ get_changed_events (ECalBackendExchange *cbex, const char *since)
 
 		uid = g_hash_table_lookup (attachments, hrefs->pdata[i]);
 
-		add_ical (cbex, hrefs->pdata[i], modtime, uid, body, length);
+		add_ical (cbex, hrefs->pdata[i], modtime, uid, body, length, 0);
 		g_free (body);
 	}
 
@@ -521,7 +495,7 @@ add_timezone_cb (icalparameter *param, void *data)
 	struct _cb_data *cbdata = (struct _cb_data *) data;
 	icalcomponent *vtzcomp;
 	const char *tzid;
-	char *izone;
+	char *izone = NULL;
 
 	g_return_if_fail (cbdata != NULL);
 
@@ -540,118 +514,38 @@ add_timezone_cb (icalparameter *param, void *data)
 		icalcomponent_add_component (cbdata->vcal_comp, vtzcomp);
 }
 
-static char *
-build_msg ( ECalBackendExchange *cbex, ECalComponent *comp, const char *subject, const char **boundary)
+gboolean
+check_for_send_options (icalcomponent *icalcomp, E2kProperties *props)
 {
-	CamelMimeMessage *msg;
-	CamelStreamMem *content;
-	CamelMimePart *mime_part;
-	CamelDataWrapper *dw, *wrapper;
-	CamelMultipart *multipart;
-	CamelInternetAddress *from;
-	CamelStream *stream;
-	CamelContentType *type;
-	char *buffer = NULL, *cid;
-	char *from_name, *from_email;
-	GSList *attach_list = NULL, *l, *new_attach_list = NULL;
-	char *fname, *filename = NULL, *file_contents = NULL, *dest_url, *dest_file;
-	int fd, len = 0;
-	struct stat sb;
+	icalproperty *icalprop;
+	gboolean exists = FALSE;
+	const char *x_name, *x_val;
 
-	e_cal_backend_exchange_get_from (E_CAL_BACKEND_SYNC (cbex), comp, &from_name, &from_email);
-
-	e_cal_component_get_attachment_list (comp, &attach_list);
-	for (l = attach_list; l ; l = l->next){
-		fname = (char *)l->data;
-	
-		filename = g_strrstr (fname, "/") + 1;
-
-		fd = open (fname, O_RDONLY);
-		if (fd < 0) {
-			d(printf ("Could not open the attachment file\n"));
-			continue;
+	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);	
+	while (icalprop && !exists) {
+		x_name = icalproperty_get_x_name (icalprop);
+		if (!strcmp(x_name, "X-EVOLUTION-OPTIONS-TRACKINFO")) {
+			exists = TRUE;
+			x_val = icalproperty_get_x (icalprop);
+			switch (atoi (x_val)) {
+				case EX_ALL: /* Track if delivered and opened */
+				case EX_READ_AND_DELIVERED: /* Track if delivered and opened */
+					e2k_properties_set_int (props, 
+					PR_READ_RECEIPT_REQUESTED, 1);
+					/* Fall Through */
+				case EX_DELIVERED_RECEIPTS : /* Track if delivered */
+					e2k_properties_set_int (props, 
+					PR_ORIGINATOR_DELIVERY_REPORT_REQUESTED, 1);
+					break;
+				default : /* None */
+					exists = FALSE;
+					break; 
+			}
 		}
-		if (fstat (fd, &sb) < 0) {
-			d(printf ("fstat of attachment file failed\n"));
-		}
-		len = sb.st_size;
-
-		file_contents = g_malloc0 (len + 1);
-
-		if (len > 0 && camel_read (fd, file_contents, len) < 0) {
-			d(printf ("reading from the attachment file failed\n"));
-			close (fd);
-		}
-		file_contents [len] = '\0';
-		close (fd);
-
-		/* Write it to our local exchange store in .evolution */
-		dest_file = e_folder_exchange_get_storage_file (cbex->folder, filename);
-		fd = open (dest_file, O_RDWR | O_CREAT | O_TRUNC, 0600);
-		if (fd < 0) {
-			d(printf ("open of destination file for attachments failed\n"));
-		}
-		
-		if (camel_write (fd, file_contents, len) < 0)
-			d(printf ("camel write to attach file failed\n"));
-		/* FIXME : Add a ATTACH:CID:someidentifier here */
-		dest_url = g_strconcat ("file:///", dest_file, NULL);
-		new_attach_list = g_slist_append (new_attach_list, dest_url);
-		g_free (dest_file);
-	}
-	e_cal_component_set_attachment_list (comp, new_attach_list);
-
-
-	if (len) {
-		msg = camel_mime_message_new ();
-
-		multipart = camel_multipart_new ();
-
-		/* Headers */
-		camel_mime_message_set_subject (msg, subject);
-
-		from = camel_internet_address_new ();
-		camel_internet_address_add (from, from_name, from_email);
-		camel_mime_message_set_from (msg, from);
-		camel_object_unref (from);
-		
-		/* Content */
-		stream = camel_stream_mem_new_with_buffer (file_contents, len);
-		wrapper = camel_data_wrapper_new ();
-		camel_data_wrapper_construct_from_stream (wrapper, stream);
-		camel_object_unref (stream);
-
-		type = camel_content_type_new ("text", "plain");
-		camel_content_type_set_param (type, "name", "\"test-attach.txt\"");
-		camel_data_wrapper_set_mime_type_field (wrapper, type);
-		camel_content_type_unref (type);
-
-		//mime_part = CAMEL_MIME_PART (msg);
-		mime_part = camel_mime_part_new ();
-
-		camel_medium_set_content_object (CAMEL_MEDIUM (mime_part), wrapper);
-		camel_mime_part_set_filename (mime_part, filename);
-		camel_mime_part_set_encoding (mime_part, CAMEL_TRANSFER_ENCODING_BASE64);
-		cid = camel_header_msgid_generate ();
-		camel_mime_part_set_content_id (mime_part, cid);	
-		camel_mime_part_set_description (mime_part, filename);
-		camel_mime_part_set_disposition (mime_part, "attachment");
-		camel_multipart_set_boundary (multipart, NULL);
-		*boundary = camel_multipart_get_boundary (multipart);
-		camel_multipart_add_part (multipart, mime_part);
-		camel_object_unref (mime_part);
-		camel_medium_set_content_object (CAMEL_MEDIUM (msg), CAMEL_DATA_WRAPPER (multipart));
-		camel_object_unref (multipart);
-
-		content = (CamelStreamMem *)camel_stream_mem_new();
-		dw = camel_medium_get_content_object (CAMEL_MEDIUM (msg));
-		camel_data_wrapper_decode_to_stream(dw, (CamelStream *)content);
-		buffer = g_memdup (content->buffer->data, content->buffer->len);
-		d(printf ("|| Buffer: \n%s\n", buffer));
-		g_free (cid);
+		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
 	}
 
-	return buffer;
+	return exists;
 }
 
 static ECalBackendSyncStatus
@@ -663,21 +557,22 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	icalcomponent *icalcomp, *real_icalcomp;
 	icalcomponent_kind kind;
 	icalproperty *icalprop;
-	const char *comp_uid;
+	const char *temp_comp_uid;
 	char *busystatus, *insttype, *allday, *importance, *lastmod;
 	struct icaltimetype current, startt;
 	char *location, *ru_header;
 	ECalComponent *comp;
 	char *body, *body_crlf, *msg;
 	char *from, *date;
-	const char *x_name, *x_val;
 	const char *summary;
 	char *attach_body = NULL;
 	char *attach_body_crlf = NULL;
-	const char *boundary;
+	char *boundary = NULL;
 	E2kHTTPStatus http_status;
+	E2kProperties *props = e2k_properties_new ();
+	E2kContext *e2kctx;
 	struct _cb_data *cbdata;
-	GSList *categories;
+	gboolean send_options;
 	
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 	
@@ -705,7 +600,11 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
 	
-	comp_uid = icalcomponent_get_uid (icalcomp);
+	temp_comp_uid = icalcomponent_get_uid (icalcomp);
+	if (!temp_comp_uid) {
+		icalcomponent_free (icalcomp);
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
 	#if 0
 	if (lookup_component (E_CAL_BACKEND_EXCHANGE (cbexc), comp_uid))
 	{
@@ -715,26 +614,8 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	#endif
 	
 	/* Send options */
-	icalprop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);	
-	while (icalprop) {
-		x_name = icalproperty_get_x_name (icalprop);
-		if (!strcmp(x_name, "X-EVOLUTION-OPTIONS-TRACKINFO")) {
-			x_val = icalproperty_get_x (icalprop);
-			switch (atoi (x_val)) {
-				case 1: /* Track if delivered */
-					break;
-				case 2: /* Track if opened */
-					break;
-				case 3: /* Track if delivered and opened */
-					break;
-				default : /* None */
-					break; 
-			}
-		}
-		icalprop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
-	}
-
-
+	send_options = check_for_send_options (icalcomp, props);
+	
 	/*set X-MICROSOFT-CDO properties*/
 	busystatus = "BUSY";
 	icalprop = icalcomponent_get_first_property (icalcomp, 
@@ -797,7 +678,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	
 	/* Check for attachments */
 	if (e_cal_component_has_attachments (comp)) {
-		printf ("This comp has attachments !!\n");
+		d(printf ("This comp has attachments !!\n"));
 		attach_body = build_msg (E_CAL_BACKEND_EXCHANGE (cbexc), comp, summary, &boundary);
 		attach_body_crlf = e_cal_backend_exchange_lf_to_crlf (attach_body);	
 	}
@@ -857,6 +738,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 				       "\r\n%s\r\n%s", summary, date, boundary,
 				       from ? from : "Evolution", boundary,
 				       body_crlf, attach_body_crlf);
+		g_free (boundary);
 
 	} else {
 		msg = g_strdup_printf ("Subject: %s\r\n"
@@ -879,6 +761,11 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 						NULL, NULL, "message/rfc822", 
 						msg, strlen(msg), &location, &ru_header);	
 
+	if ((http_status == E2K_HTTP_CREATED) && send_options) {
+		e2kctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);
+		http_status = e2k_context_proppatch (e2kctx, NULL, location, props, FALSE, NULL);
+	}
+
 	g_free (date);
 	g_free (from);
 	g_free (body_crlf);
@@ -886,7 +773,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	icalcomponent_free (cbdata->vcal_comp); // not sure
 	g_free (cbdata);
 
-	if (http_status != E2K_HTTP_CREATED) {
+	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status)) {
 		g_object_unref (comp);
 		g_free (location);
 		g_free (lastmod);
@@ -895,15 +782,13 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 
 	/*add object*/
 	e_cal_backend_exchange_add_object (E_CAL_BACKEND_EXCHANGE (cbexc), location, lastmod, icalcomp);
-	
-	e_cal_component_get_categories_list (comp, &categories);
-	e_cal_backend_ref_categories (E_CAL_BACKEND (cbexc), categories);
-	e_cal_component_free_categories_list (categories);
-	
+		
 	g_object_unref (comp);
 	g_free (lastmod);
 	g_free (location);
-	
+	e2k_properties_free (props);
+
+	*uid = g_strdup (temp_comp_uid);	
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -1030,6 +915,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	const char *comp_uid;
 	char *updated_ecomp_str, *real_comp_str;
 	char *body, *body_crlf, *msg;
+	char *attach_body = NULL;
+	char *attach_body_crlf = NULL;
+	char *boundary = NULL;
 	struct icaltimetype last_modified;
 	icalcomponent_kind kind;
 	ECalComponentDateTime dt;
@@ -1038,8 +926,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	E2kHTTPStatus http_status;
 	char *from, *date;
 	const char *summary, *new_href;
+	gboolean send_options;
 	E2kContext *ctx;
-	GSList *categories;
+	E2kProperties *props = e2k_properties_new ();
 
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 
@@ -1070,6 +959,11 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	}
 	
+	/* Fetch summary */
+	summary = icalcomponent_get_summary (icalcomp);
+	if (!summary)
+		summary = "";
+	
 	updated_ecomp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (updated_ecomp, icalcomp);
 
@@ -1077,7 +971,13 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	
 	last_modified = icaltime_from_timet (time (NULL), 0);
 	e_cal_component_set_last_modified (updated_ecomp, &last_modified);
-	
+
+	if (e_cal_component_has_attachments (updated_ecomp)) {
+		d(printf ("This comp has attachments !!\n"));
+		attach_body = build_msg (E_CAL_BACKEND_EXCHANGE (cbexc), updated_ecomp, summary, &boundary);
+		attach_body_crlf = e_cal_backend_exchange_lf_to_crlf (attach_body);	
+	}
+
 	updated_ecomp_str = e_cal_component_get_as_string (updated_ecomp);
 	updated_icalcomp = icalparser_parse_string (updated_ecomp_str);
 	g_free (updated_ecomp_str);
@@ -1086,6 +986,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
+	/* send options */
+	send_options = check_for_send_options (updated_icalcomp, props);
+
 	/* Remove X parameters from properties */
 	/* This is specifically for X-EVOLUTION-END-DATE, 
 	   but removing anything else is probably ok too */
@@ -1151,7 +1054,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 			if (dt.tzid == NULL)
 				from_zone = icaltimezone_get_utc_timezone ();
 			else {
-				char *izone;
+				char *izone = NULL;
 				get_timezone (backend, cal, dt.tzid, &izone);
 				from_zone = icalcomponent_get_timezone (icalcomponent_new_from_string (izone),
 											dt.tzid);
@@ -1198,24 +1101,44 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	date = e_cal_backend_exchange_make_timestamp_rfc822 (time (NULL));
 	from = e_cal_backend_exchange_get_from_string (backend, updated_ecomp);	
 	
-	summary = icalcomponent_get_summary (real_icalcomp);
-	if (!summary)
-		summary = "";
+	if (attach_body) {
+		msg = g_strdup_printf ("Subject: %s\r\n"
+				       "Date: %s\r\n"
+				       "MIME-Version: 1.0\r\n"
+				       "Content-Type: multipart/mixed;\r\n"
+				       "\tboundary=\"%s\";\r\n"
+				       "X-MS_Has-Attach: yes\r\n"
+				       "From: %s\r\n"
+					"\r\n--%s\r\n"
+				       "content-class: urn:content-classes:appointment\r\n"
+				       "Content-Type: text/calendar;\r\n"
+				       "\tmethod=REQUEST;\r\n"
+				       "\tcharset=\"utf-8\"\r\n"
+				       "Content-Transfer-Encoding: 8bit\r\n"
+				       "Importance: normal\r\n"
+				       "Priority: normal\r\n"
+				       "\r\n%s\r\n%s", summary, date, boundary,
+				       from ? from : "Evolution", boundary,
+				       body_crlf, attach_body_crlf);
+		g_free (boundary);
 
-	msg = g_strdup_printf ("Subject: %s\r\n"
-			       "Date: %s\r\n"
-			       "MIME-Version: 1.0\r\n"
-			       "Content-Type: text/calendar;\r\n"
-			       "\tmethod=REQUEST;\r\n"
-			       "\tcharset=\"utf-8\"\r\n"
-			       "Content-Transfer-Encoding: 8bit\r\n"
-			       "content-class: urn:content-classes:appointment\r\n"
-			       "Importance: normal\r\n"
-			       "Priority: normal\r\n"
-			       "From: %s\r\n"
-			       "\r\n%s", summary, date,
-			       from ? from : "Evolution",
-			       body_crlf);
+	} else {
+
+		msg = g_strdup_printf ("Subject: %s\r\n"
+				       "Date: %s\r\n"
+				       "MIME-Version: 1.0\r\n"
+				       "Content-Type: text/calendar;\r\n"
+				       "\tmethod=REQUEST;\r\n"
+				       "\tcharset=\"utf-8\"\r\n"
+				       "Content-Transfer-Encoding: 8bit\r\n"
+				       "content-class: urn:content-classes:appointment\r\n"
+				       "Importance: normal\r\n"
+				       "Priority: normal\r\n"
+				       "From: %s\r\n"
+				       "\r\n%s", summary, date,
+				       from ? from : "Evolution",
+				       body_crlf);
+	}
 	
 	g_free (date);
 	g_free (from);
@@ -1225,16 +1148,6 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	e_cal_component_set_icalcomponent (cached_ecomp, icalcomponent_new_clone (ecomp->icomp));
 	*old_object = e_cal_component_get_as_string (cached_ecomp);
 	
-	/*unref the old set of categories*/
-	e_cal_component_get_categories_list (cached_ecomp, &categories);
-	e_cal_backend_unref_categories (E_CAL_BACKEND (cbexc), categories);
-	e_cal_component_free_categories_list (categories);
-	
-	/*ref the new set of categories*/
-	e_cal_component_get_categories_list (real_ecomp, &categories);
-	e_cal_backend_ref_categories (E_CAL_BACKEND (cbexc), categories);
-	e_cal_component_free_categories_list (categories);
-
 	ctx = exchange_account_get_context (E_CAL_BACKEND_EXCHANGE (cbexc)->account);	
 	
 	/* PUT the iCal object in the Exchange server */
@@ -1246,6 +1159,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	http_status = e2k_context_put (ctx, NULL, new_href, "message/rfc822",
 				       		msg, strlen (msg), NULL);
 
+	if ((E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status)) && send_options)
+		http_status = e2k_context_proppatch (ctx, NULL, new_href, props, FALSE, NULL);
+
 	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (http_status))
 		e_cal_backend_exchange_modify_object (E_CAL_BACKEND_EXCHANGE (cbexc), 
 							real_icalcomp, mod);
@@ -1256,6 +1172,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	g_object_unref (cached_ecomp);
 	icalcomponent_free (cbdata->vcal_comp);
 	g_free (cbdata);
+	e2k_properties_free (props);
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -1270,7 +1187,6 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 	E2kHTTPStatus status;
 	E2kContext *ctx;
 	ECalComponent *comp;
-	GSList *categories;
 	char *calobj, *obj;
 	struct icaltimetype time_rid;
 	ECalBackendSyncStatus ebs_status;
@@ -1291,7 +1207,8 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 		
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->icomp));
-	*object = e_cal_component_get_as_string (comp);
+	if(object)
+		*object = e_cal_component_get_as_string (comp);
 	
 	switch (mod) {
 		
@@ -1300,9 +1217,6 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 			
 			status = e2k_context_delete (ctx, NULL, ecomp->href);			
 			if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {		
-				e_cal_component_get_categories_list (comp, &categories);
-				e_cal_backend_unref_categories (E_CAL_BACKEND (cbexc), categories);
-				e_cal_component_free_categories_list (categories);
 				
 				if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) {
 						g_object_unref (comp);
@@ -1417,6 +1331,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		g_object_unref (comp);
 	}
 	g_list_free (comps);
+	return status;
 		
  error:	
 	if (comp)
@@ -1868,7 +1783,7 @@ discard_alarm (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 
 	ecomp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (ecomp, ecbexcomp->icomp);
+	e_cal_component_set_icalcomponent (ecomp, icalcomponent_new_clone (ecbexcomp->icomp));
 	if (!e_cal_component_has_recurrences (ecomp))
 	{
 		e_cal_component_remove_alarm (ecomp, auid);
